<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Board customize action controller
 *
 * @author Egor Bulgakov <egor.bulgakov@gmail.com>
 * @package ow.ow_plugins.board.controllers
 * @since 1.0
 */
class BOARD_CTRL_Customize extends OW_ActionController
{

    /**
     * Controller's default action
     */
    public function index()
    {
        $boardService = BOARD_BOL_BoardService::getInstance();

        $sectionGroupList = $boardService->getCustomSectionGroupList();
        $isModerator = OW::getUser()->isAuthorized('board');

        if ( !$isModerator )
        {
            throw new Redirect404Exception();
        }

        $this->assign('sectionGroupList', $sectionGroupList);
        
        $plugin = OW::getPluginManager()->getPlugin('board');

        //add langs for javascript
        OW::getLanguage()->addKeyForJs('board', 'delete_section_confirm');
        OW::getLanguage()->addKeyForJs('board', 'delete_group_confirm');
        OW::getLanguage()->addKeyForJs('board', 'add_new_board_title');
        OW::getLanguage()->addKeyForJs('board', 'edit_section_title');
        OW::getLanguage()->addKeyForJs('board', 'edit_group_title');
        
        OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('base')->getStaticJsUrl() . 'jquery-ui-1.8.9.custom.min.js');
        OW::getDocument()->addScript($plugin->getStaticJsUrl() . 'board.js');

        $sortSectionOrderUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'sortSectionOrder');
        $sortGroupOrderUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'sortGroupOrder');
        $deleteSectionUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'deleteSection');
        $deleteGroupUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'deleteGroup');
        $getSectionUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'getSection');
        $getGroupUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'getGroup');
        $editSectionUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'editSection');
        $editGroupUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'editGroup');
        $addBoardUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'addBoard');
        $suggestSectionUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'suggestSection');
        $boardIndexUrl = OW::getRouter()->urlForRoute('board-default');

        $onloadJs = "
			BoardCustomize.sortSectionOrderUrl = '$sortSectionOrderUrl';
			BoardCustomize.sortGroupOrderUrl = '$sortGroupOrderUrl';
			BoardCustomize.deleteSectionUrl = '$deleteSectionUrl';
			BoardCustomize.deleteGroupUrl = '$deleteGroupUrl';
			BoardCustomize.suggestSectionUrl = '$suggestSectionUrl';
			BoardCustomize.getSectionUrl = '$getSectionUrl';
			BoardCustomize.getGroupUrl = '$getGroupUrl';
			BoardCustomize.boardIndexUrl = '$boardIndexUrl';
			
        	BoardCustomize.construct();
        	";
        OW::getDocument()->addOnloadScript($onloadJs);

        $addBoardForm = $this->generateAddBoardForm($addBoardUrl);
        $this->addForm($addBoardForm);

        $editSectionForm = $this->generateEditSectionForm($editSectionUrl);
        $this->addForm($editSectionForm);

        $editGroupForm = $this->generateEditGroupForm($editGroupUrl);
        $this->addForm($editGroupForm);

        OW::getDocument()->setHeading(OW::getLanguage()->text('board', 'board'));
        OW::getDocument()->setHeadingIconClass('ow_ic_board');
        
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'board', 'board');
    }

    /**
     * This action updates section orders called by ajax request
     */
    public function sortSectionOrder()
    {
        $isModerator = OW::getUser()->isAuthorized('board');

        if ( OW::getRequest()->isAjax() && $_POST['data'] && $isModerator )
        {
            $boardService = BOARD_BOL_BoardService::getInstance();

            $postData = array();
            parse_str($_POST['data'], $postData);

            if ( !$postData['section'] )
            {
                return false;
            }

            $sectionOrderList = array();
            $sectionOrderList = array_flip($postData['section']);

            $sectionDtoList = $boardService->findSectionList();

            foreach ( $sectionDtoList as $sectionDto )
            {
                $sectionDto->order = $sectionOrderList[$sectionDto->id] + 1;
                $boardService->saveOrUpdateSection($sectionDto);
            }
        }

        exit();
    }

    /**
     * This action updates group orders called by ajax request
     */
    public function sortGroupOrder()
    {
        $isModerator = OW::getUser()->isAuthorized('board');

        if ( OW::getRequest()->isAjax() && $_POST['data'] && $isModerator )
        {
            $boardService = BOARD_BOL_BoardService::getInstance();

            $sectionGroupList = json_decode($_POST['data'], true);

            foreach ( $sectionGroupList as $sectionGroup )
            {
                if ( !$sectionGroup['order'] )
                {
                    continue;
                }

                $sectionId = (int) $sectionGroup['sectionId'];

                $postData = array();
                parse_str($sectionGroup['order'], $postData);

                $groupOrderList = array();
                $groupOrderList = array_flip($postData['group']);

                $groupDtoList = $boardService->findGroupByIdList($postData['group']);
                $groupDto = new BOARD_BOL_Group();
                foreach ( $groupDtoList as $groupDto )
                {
                    $groupDto->sectionId = $sectionId;
                    $groupDto->order = $groupOrderList[$groupDto->id] + 1;

                    $boardService->saveOrUpdateGroup($groupDto);
                }
            }
        }

        exit();
    }

    /**
     * This action deletes section called by ajax request
     */
    public function deleteSection()
    {
        $isModerator = OW::getUser()->isAuthorized('board');

        $sectionId = (int) $_POST['sectionId'];

        if ( OW::getRequest()->isAjax() && $sectionId && $isModerator )
        {
            $boardService = BOARD_BOL_BoardService::getInstance();

            $boardService->deleteSection($sectionId);

            echo 1;
        }

        exit();
    }

    /**
     * This action deletes group called by ajax request
     */
    public function deleteGroup()
    {
        $isModerator = OW::getUser()->isAuthorized('board');

        $groupId = (int) $_POST['groupId'];

        if ( OW::getRequest()->isAjax() && $groupId && $isModerator )
        {
            $boardService = BOARD_BOL_BoardService::getInstance();

            $boardService->deleteGroup($groupId);

            echo 1;
        }

        exit();
    }

    /**
     * This action adds section and group called by ajax request
     */
    public function addBoard()
    {
        $isModerator = OW::getUser()->isAuthorized('board');

        if ( OW::getRequest()->isAjax() && $_POST && $isModerator )
        {
            $groupName = trim($_POST['group-name']);
            $sectionName = trim($_POST['section']);
            $groupDescription = trim($_POST['description']);
            $isPrivate = $_POST['is-private'] == 'on';
            $roles = !empty($_POST['roles']) ? $_POST['roles'] : null;

            if ( !$groupName || !$sectionName || !$groupDescription )
            {
                exit();
            }

            $boardService = BOARD_BOL_BoardService::getInstance();

            $sectionDto = $boardService->getSection($sectionName, 0);

            //create board section
            if ( $sectionDto === null )
            {
                $sectionDto = new BOARD_BOL_Section();
                $sectionDto->name = $sectionName;
                $sectionDto->order = $boardService->getNewSectionOrder();

                $boardService->saveOrUpdateSection($sectionDto);
            }

            //create board group
            $groupDto = new BOARD_BOL_Group();

            $groupDto->sectionId = $sectionDto->id;
            $groupDto->name = $groupName;
            $groupDto->description = $groupDescription;
            $groupDto->order = $boardService->getNewGroupOrder($sectionDto->id);
            $groupDto->isPrivate = (bool) $isPrivate;
            $groupDto->roles = count($roles) ? json_encode($roles) : null;

            $boardService->saveOrUpdateGroup($groupDto);

            echo true;
            exit();
        }
    }

    /**
     * This action suggests sections called by ajax request
     */
    public function suggestSection()
    {
        if ( OW::getRequest()->isAjax() && isset($_GET['q']) )
        {
            $sectionName = trim($_GET['q']);

            $boardService = BOARD_BOL_BoardService::getInstance();
            $sectionDtoList = $boardService->suggestSection($sectionName);

            if ( $sectionDtoList )
            {
                foreach ( $sectionDtoList as $sectionDto )
                {
                    echo "$sectionDto->name\t$sectionDto->id\n";
                }
            }
        }

        exit();
    }

    /**
     * This action edits section called by ajax request
     */
    public function editSection()
    {
        $isModerator = OW::getUser()->isAuthorized('board');

        if ( OW::getRequest()->isAjax() && $_POST && $isModerator )
        {
            $sectionName = trim($_POST['section-name']);
            $sectionId = (int) $_POST['section-id'];

            if ( !$sectionName || !$sectionId )
            {
                exit();
            }

            $boardService = BOARD_BOL_BoardService::getInstance();
            $sectionDto = $boardService->findSectionById($sectionId);

            if ( $sectionDto === null )
            {
                exit();
            }

            $sectionDto->name = $sectionName;

            $boardService->saveOrUpdateSection($sectionDto);

            echo true;
            exit();
        }
    }

    /**
     * This action edits group called by ajax request
     */
    public function editGroup()
    {
        $isModerator = OW::getUser()->isAuthorized('board');

        if ( OW::getRequest()->isAjax() && $_POST && $isModerator )
        {
            $groupId = (int) $_POST['group-id'];
            $groupName = trim($_POST['group-name']);
            $groupDescription = trim($_POST['description']);
            $isPrivate = $_POST['is-private'] == 'on';
            $roles = !empty($_POST['roles']) ? $_POST['roles'] : null;

            if ( !$groupId || !$groupName || !$groupDescription )
            {
                exit();
            }

            $boardService = BOARD_BOL_BoardService::getInstance();
            $groupDto = $boardService->findGroupById($groupId);

            if ( $groupDto === null )
            {
                exit();
            }

            $groupDto->name = $groupName;
            $groupDto->description = $groupDescription;
            $groupDto->isPrivate = (bool) $isPrivate;
            $groupDto->roles = count($roles) ? json_encode($roles) : null;

            $boardService->saveOrUpdateGroup($groupDto);

            echo true;
            exit();
        }
    }

    /**
     * This action returns section called by ajax request
     */
    public function getSection()
    {
        if ( OW::getRequest()->isAjax() && $_POST['sectionId'] )
        {
            $sectionId = (int) $_POST['sectionId'];

            $boardService = BOARD_BOL_BoardService::getInstance();
            $sectionDto = $boardService->findSectionById($sectionId);

            echo json_encode($sectionDto);
        }

        exit();
    }

    /**
     * This action returns group called by ajax request
     */
    public function getGroup()
    {
        if ( OW::getRequest()->isAjax() && $_POST['groupId'] )
        {
            $groupId = (int) $_POST['groupId'];

            $boardService = BOARD_BOL_BoardService::getInstance();
            $groupDto = $boardService->findGroupById($groupId);

            $groupDto->roles = json_decode($groupDto->roles, true);
            echo json_encode($groupDto);
        }

        exit();
    }

    /**
     * Generates add board form
     * 
     * @param string $action
     * @return Form
     */
    private function generateAddBoardForm( $action )
    {
        $language = OW::getLanguage();
        $form = new Form('add-board-form');

        $form->setAction($action);

        $groupName = new TextField('group-name');
        $groupName->setRequired(true);
        $groupName->addValidator(new StringValidator(1, 255));
        $form->addElement($groupName);

        $sectionField = new SuggestField('section');
        $sectionField->setRequired(true);
        $sectionField->setMinChars(1);
        $responderUrl = OW::getRouter()->urlFor('BOARD_CTRL_Customize', 'suggestSection');
        $sectionField->setResponderUrl($responderUrl);

        $firstSection = BOARD_BOL_BoardService::getInstance()->getFirstSection();
        if ( $firstSection )
        {
            $sectionField->setValue($firstSection->name);
        }

        $form->addElement($sectionField->setLabel($language->text('board', 'section')));

        $description = new Textarea('description');
        $description->setRequired(true);
        $description->addValidator(new StringValidator(1, 50000));
        $form->addElement($description);

        $isPrivate = new CheckboxField('is-private');
        $form->addElement($isPrivate);
        
        $roles = new CheckboxGroup('roles');
        $authService = BOL_AuthorizationService::getInstance();
        $roleList = $authService->getRoleList();
        $options = array();
        foreach ( $roleList as $role )
        {
            $options[$role->id] = $authService->getRoleLabel($role->name);
        }
        $roles->addOptions($options);
        $roles->setColumnCount(2);
        $form->addElement($roles);
        
        $submit = new Submit('add');
        $submit->setValue(OW::getLanguage()->text('board', 'add_new_board_submit'));
        $form->addElement($submit);

        $form->setAjax(true);

        return $form;
    }

    /**
     * Generates edit section form
     * 
     * @param string $action
     * @return Form
     */
    private function generateEditSectionForm( $action )
    {
        $form = new Form('edit-section-form');

        $form->setAction($action);

        $sectionName = new TextField('section-name');
        $sectionName->setRequired(true);
        $sectionName->addValidator(new StringValidator(1, 255));
        $form->addElement($sectionName);

        $sectionId = new HiddenField('section-id');
        $sectionId->setRequired(true);
        $form->addElement($sectionId);

        $submit = new Submit('save');
        $submit->setValue(OW::getLanguage()->text('board', 'edit_section_btn'));
        $form->addElement($submit);

        $form->setAjax(true);

        return $form;
    }

    /**
     * Generates edit group form
     * 
     * @param string $action
     * @return Form
     */
    private function generateEditGroupForm( $action )
    {
        $form = new Form('edit-group-form');

        $form->setAction($action);

        $groupName = new TextField('group-name');
        $groupName->setRequired(true);
        $groupName->addValidator(new StringValidator(1, 255));
        $form->addElement($groupName);

        $description = new Textarea('description');
        $description->setRequired(true);
        $description->addValidator(new StringValidator(1, 50000));
        $form->addElement($description);

        $groupId = new HiddenField('group-id');
        $groupId->setRequired(true);
        $form->addElement($groupId);
        
        $isPrivate = new CheckboxField('is-private');
        $form->addElement($isPrivate);
        
        $roles = new CheckboxGroup('roles');
        $authService = BOL_AuthorizationService::getInstance();
        $roleList = $authService->getRoleList();
        $options = array();
        foreach ( $roleList as $role )
        {
            $options[$role->id] = $authService->getRoleLabel($role->name);
        }
        $roles->addOptions($options);
        $roles->setColumnCount(2);
        $form->addElement($roles);

        $submit = new Submit('save');
        $submit->setValue(OW::getLanguage()->text('board', 'edit_group_btn'));
        $form->addElement($submit);

        $form->setAjax(true);

        return $form;
    }
}