<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2011, Oxwall Foundation
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Oxwall Foundation nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
$plugin = OW::getPluginManager()->getPlugin('board');

OW::getAutoloader()->addClass('BoardSelectBox', $plugin->getRootDir() . 'classes' . DS . 'board_select_box.php');
OW::getAutoloader()->addClass('BoardStringValidator', $plugin->getRootDir() . 'classes' . DS . 'board_string_validator.php');

OW::getRouter()->addRoute(new OW_Route('board-default', 'board', 'BOARD_CTRL_Index', 'index'));
OW::getRouter()->addRoute(new OW_Route('customize-default', 'board/customize', 'BOARD_CTRL_Customize', 'index'));
OW::getRouter()->addRoute(new OW_Route('group-default', 'board/:groupId', 'BOARD_CTRL_Group', 'index'));
OW::getRouter()->addRoute(new OW_Route('topic-default', 'board/topic/:topicId', 'BOARD_CTRL_Topic', 'index'));

OW::getRouter()->addRoute(new OW_Route('add-topic-default', 'board/addTopic', 'BOARD_CTRL_AddTopic', 'index'));
OW::getRouter()->addRoute(new OW_Route('add-topic', 'board/addTopic/:groupId', 'BOARD_CTRL_AddTopic', 'index'));

OW::getRouter()->addRoute(new OW_Route('sticky-topic', 'board/stickyTopic/:topicId/:page', 'BOARD_CTRL_Topic', 'stickyTopic'));
OW::getRouter()->addRoute(new OW_Route('lock-topic', 'board/lockTopic/:topicId/:page', 'BOARD_CTRL_Topic', 'lockTopic'));
OW::getRouter()->addRoute(new OW_Route('delete-topic', 'board/deleteTopic/:topicId', 'BOARD_CTRL_Topic', 'deleteTopic'));
OW::getRouter()->addRoute(new OW_Route('get-post', 'board/getPost/:postId', 'BOARD_CTRL_Topic', 'getPost'));
OW::getRouter()->addRoute(new OW_Route('edit-post', 'board/edit-post/:id', 'BOARD_CTRL_EditPost', 'index'));
OW::getRouter()->addRoute(new OW_Route('edit-topic', 'board/edit-topic/:id', 'BOARD_CTRL_EditTopic', 'index'));
OW::getRouter()->addRoute(new OW_Route('move-topic', 'board/moveTopic', 'BOARD_CTRL_Topic', 'moveTopic'));
OW::getRouter()->addRoute(new OW_Route('subscribe-topic', 'board/subscribe-topic/:id', 'BOARD_CTRL_Topic', 'subscribeTopic'));
OW::getRouter()->addRoute(new OW_Route('unsubscribe-topic', 'board/unsubscribe-topic/:id', 'BOARD_CTRL_Topic', 'unsubscribeTopic'));

OW::getRouter()->addRoute(new OW_Route('add-post', 'board/addPost/:topicId', 'BOARD_CTRL_Topic', 'addPost'));
OW::getRouter()->addRoute(new OW_Route('delete-post', 'board/deletePost/:topicId/:postId', 'BOARD_CTRL_Topic', 'deletePost'));
OW::getRouter()->addRoute(new OW_Route('board_delete_attachment', 'board/deleteAttachment/', 'BOARD_CTRL_Topic', 'ajaxDeleteAttachment'));
OW::getRouter()->addRoute(new OW_Route('board_admin_config', 'admin/plugins/board', 'BOARD_CTRL_Admin', 'index'));
OW::getRouter()->addRoute(new OW_Route('board_uninstall', 'admin/board/uninstall', 'BOARD_CTRL_Admin', 'uninstall'));
OW::getRouter()->addRoute(new OW_Route('board_search', 'board/search/', 'BOARD_CTRL_Search', 'inBoards'));
OW::getRouter()->addRoute(new OW_Route('board_search_group', 'board/:groupId/search/', 'BOARD_CTRL_Search', 'inGroup'));
OW::getRouter()->addRoute(new OW_Route('board_search_topic', 'board/topic/:topicId/search/', 'BOARD_CTRL_Search', 'inTopic'));

function board_elst_add_new_content_item( BASE_CLASS_EventCollector $event )
{
    if ( !OW::getUser()->isAuthorized('board', 'edit') )
    {
        return;
    }

    $resultArray = array(
        BASE_CMP_AddNewContent::DATA_KEY_ICON_CLASS => 'ow_ic_files',
        BASE_CMP_AddNewContent::DATA_KEY_URL => OW::getRouter()->urlForRoute('add-topic-default'),
        BASE_CMP_AddNewContent::DATA_KEY_LABEL => OW::getLanguage()->text('board', 'discussion')
    );

    $event->add($resultArray);
}
OW::getEventManager()->bind(BASE_CMP_AddNewContent::EVENT_NAME, 'board_elst_add_new_content_item');

function board_delete_user_content( OW_Event $event )
{
    $params = $event->getParams();

    if ( !isset($params['deleteContent']) || !(bool) $params['deleteContent'] )
    {
        return;
    }

    $userId = (int) $params['userId'];

    if ( $userId > 0 )
    {
        $boardService = BOARD_BOL_BoardService::getInstance();

        $boardService->deleteUserTopics($userId);
        $boardService->deleteUserPosts($userId);
    }
}
OW::getEventManager()->bind(OW_EventManager::ON_USER_UNREGISTER, 'board_delete_user_content');

function board_create_section( OW_Event $event )
{
    $params = $event->getParams();

    if ( !isset($params['name']) || !isset($params['entity']) || !isset($params['isHidden']) )
    {
        return;
    }

    $board_service = BOARD_BOL_BoardService::getInstance();

    $sectionDto = $board_service->findSectionByEntity($params['entity']);

    if ( !isset($sectionDto) )
    {
        $sectionDto = new BOARD_BOL_Section();
        $sectionDto->name = $params['name'];
        $sectionDto->entity = $params['entity'];
        $sectionDto->isHidden = $params['isHidden'];
        $sectionDto->order = $board_service->getNewSectionOrder();

        $board_service->saveOrUpdateSection($sectionDto);
    }

    try
    {
        OW::getAuthorization()->addAction($params['entity'], 'add_topic');
    }
    catch ( Exception $e )
    {

    }
}
OW::getEventManager()->bind('board.create_section', 'board_create_section');

function board_delete_section( OW_Event $event )
{
    $params = $event->getParams();

    if ( !isset($params['name']) && !isset($params['entity']) )
    {
        return;
    }

    $board_service = BOARD_BOL_BoardService::getInstance();

    if ( isset($params['name']) )
    {
        $section = $board_service->getSection($params['name']);
    }

    if ( isset($params['entity']) )
    {
        $section = $board_service->findSectionByEntity($params['entity']);
    }

    if ( !empty($section) )
    {
        $board_service->deleteSection($section->getId());
    }
}
OW::getEventManager()->bind('board.delete_section', 'board_delete_section');

function board_add_widget( OW_Event $event )
{
    $params = $event->getParams();

    if ( !isset($params['place']) || !isset($params['section']) )
    {
        return;
    }

    try
    {
        $widgetService = BOL_ComponentAdminService::getInstance();
        $widget = $widgetService->addWidget('BOARD_CMP_LatestTopicsWidget', false);
        $placeWidget = $widgetService->addWidgetToPlace($widget, $params['place']);
        $widgetService->addWidgetToPosition($placeWidget, $params['section'], 0);
    }
    catch ( Exception $e )
    {

    }
}
OW::getEventManager()->bind('board.add_widget', 'board_add_widget');

function board_delete_widget( OW_Event $event )
{
    BOL_ComponentAdminService::getInstance()->deleteWidget('BOARD_CMP_LatestTopicsWidget');
}
OW::getEventManager()->bind('board.delete_widget', 'board_delete_widget');

function board_create_group( OW_Event $event )
{
    $params = $event->getParams();

    if ( !$params['entity'] || !isset($params['name']) || !isset($params['description']) || !isset($params['entityId']) )
    {
        return;
    }

    $boardService = BOARD_BOL_BoardService::getInstance();

    $boardGroup = $boardService->findGroupByEntityId($params['entity'], $params['entityId']);

    if ( !isset($boardGroup) )
    {
        $section = $boardService->findSectionByEntity($params['entity']);

        $boardGroup = new BOARD_BOL_Group();
        $boardGroup->sectionId = $section->getId();
        $boardGroup->order = $boardService->getNewGroupOrder($section->getId());

        $boardGroup->name = $params['name'];
        $boardGroup->description = $params['description'];
        $boardGroup->entityId = $params['entityId'];

        $boardService->saveOrUpdateGroup($boardGroup);
    }
}
OW::getEventManager()->bind('board.create_group', 'board_create_group');

function board_delete_group( OW_Event $event )
{
    $params = $event->getParams();

    if ( !isset($params['entityId']) || !isset($params['entity']) )
    {
        return;
    }

    $boardService = BOARD_BOL_BoardService::getInstance();
    $group = $boardService->findGroupByEntityId($params['entity'], $params['entityId']);

    if ( !empty($group) )
    {
        $boardService->deleteGroup($group->getId());
    }
}
OW::getEventManager()->bind('board.delete_group', 'board_delete_group');

function board_on_notify_actions( BASE_CLASS_EventCollector $e )
{
    $e->add(array(
        'section' => 'board',
        'action' => 'board-add_post',
        'sectionIcon' => 'ow_ic_board',
        'sectionLabel' => OW::getLanguage()->text('board', 'email_notifications_section_label'),
        'description' => OW::getLanguage()->text('board', 'email_notifications_setting_post'),
        'selected' => true
    ));
}
OW::getEventManager()->bind('base.notify_actions', 'board_on_notify_actions');

function board_add_post( OW_Event $e )
{
    $params = $e->getParams();

    $postId = $params['postId'];

    $boardService = BOARD_BOL_BoardService::getInstance();
    $post = $boardService->findPostById($postId);

    $userIds = BOARD_BOL_SubscriptionService::getInstance()->findTopicSubscribers($post->topicId);
    if ( empty($userIds) )
    {
        return;
    }

    $postUrl = $boardService->getPostUrl($post->topicId, $postId);
    $userAvatar = BOL_AvatarService::getInstance()->getAvatarUrl($post->userId, 1);
    $topic = $boardService->findTopicById($post->topicId);
    $topicUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $post->topicId));

    $string = OW::getLanguage()->text('board', 'email_notification_post', array(
            'userName' => BOL_UserService::getInstance()->getDisplayName($post->userId),
            'userUrl' => BOL_UserService::getInstance()->getUserUrl($post->userId),
            'postUrl' => $postUrl,
            'topicUrl' => $topicUrl,
            'title' => strip_tags($topic->title)
        ));

    $params = array(
        'plugin' => 'board',
        'action' => 'board-add_post',
        'string' => $string,
        'avatar' => $userAvatar,
        'content' => strip_tags($post->text),
        'url' => $postUrl,
        'time' => time()
    );

    foreach ( $userIds as $uid )
    {
        if ( $uid == $post->userId )
        {
            continue;
        }

        $params['userId'] = $uid;

        $event = new OW_Event('base.notify', $params);
        OW::getEventManager()->trigger($event);
    }
}
OW::getEventManager()->bind('board.add_post', 'board_add_post');

function board_ads_enabled( BASE_EventCollector $event )
{
    $event->add('board');
}
OW::getEventManager()->bind('ads.enabled_plugins', 'board_ads_enabled');


$credits = new BOARD_CLASS_Credits();
OW::getEventManager()->bind('usercredits.on_action_collect', array($credits, 'bindCreditActionsCollect'));

function board_add_auth_labels( BASE_CLASS_EventCollector $event )
{
    $language = OW::getLanguage();
    $event->add(
        array(
            'board' => array(
                'label' => $language->text('board', 'auth_group_label'),
                'actions' => array(
                    'edit' => $language->text('board', 'auth_action_label_edit'),
                    'view' => $language->text('board', 'auth_action_label_view'),
                    'subscribe' => $language->text('board', 'auth_action_label_subscribe'),
                    'move_topic_to_hidden' => $language->text('board', 'auth_action_label_move_topic_to_hidden')
                )
            )
        )
    );
}
OW::getEventManager()->bind('admin.add_auth_labels', 'board_add_auth_labels');

//Feed
function board_feed_on_entity_add( OW_Event $e )
{
    $params = $e->getParams();
    $data = $e->getData();

    if ( $params['entityType'] != 'board-topic' )
    {
        return;
    }

    $topicId = (int) $params['entityId'];
    $service = BOARD_BOL_BoardService::getInstance();
    $topicDto = $service->findTopicById($topicId);
    $postDto = $service->findTopicFirstPost($topicId);
    $groupDto = $service->findGroupById($topicDto->groupId);
    $sectionDto = $service->findSectionById($groupDto->sectionId);
    $isHidden = (bool) $sectionDto->isHidden;

    if ( $postDto === null )
    {
        return;
    }

    if ( $groupDto->isPrivate )
    {
        return;
    }

    $topicUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicDto->id));
    $content = UTIL_String::truncate(strip_tags($postDto->text), 150, '...');
    $title = UTIL_String::truncate(strip_tags($topicDto->title), 100, '...');

    $data = array(
        'features' => array('likes'),
        'ownerId' => $topicDto->userId,
        'time' => (int) $postDto->createStamp,
        'content' => '<div class="ow_newsfeed_item_content" style="float: none">
                <a class="ow_newsfeed_item_title" href="' . $topicUrl . '">' . $title . '</a>
                <div class="ow_remark ow_smallmargin">' . $content . '</div>
                <div class="ow_newsfeed_action_activity board_newsfeed_activity">[ph:activity]</div>
            </div>',
        'view' => array(
            'iconClass' => 'ow_ic_board'
        ),
        'string' => OW::getLanguage()->text('board', 'feed_activity_topic_string'),
        'toolbar' => array(array(
                'href' => $topicUrl,
                'label' => OW::getLanguage()->text('board', 'feed_toolbar_discuss')
        ))
    );

    if ( $isHidden )
    {
        $data['params']['feedType'] = $sectionDto->entity;
        $data['params']['feedId'] = $groupDto->entityId;
        $data['params']['visibility'] = 2 + 4 + 8; // Visible for follows(2), autor (4) and current feed (8)
        $data['params']['postOnUserFeed'] = false;
        $data['contextFeedType'] = $data['params']['feedType'];
        $data['contextFeedId'] = $data['params']['feedId'];
    }

    $e->setData($data);
}
OW::getEventManager()->bind('feed.on_entity_add', 'board_feed_on_entity_add');

function board_feed_on_post_add( OW_Event $e )
{
    $params = $e->getParams();

    $event = new OW_Event('feed.activity', array(
            'pluginKey' => 'board',
            'entityType' => 'board-topic',
            'entityId' => $params['topicId'],
            'userId' => $params['userId'],
            'activityType' => 'board-post',
            'activityId' => $params['postId'],
            'subscribe' => true
            ), array(
            'postId' => $params['postId'],
            'string' => OW::getLanguage()->text('board', 'feed_activity_topic_reply_string')
        ));
    OW::getEventManager()->trigger($event);
}
OW::getEventManager()->bind('board.add_post', 'board_feed_on_post_add');

function board_feed_on_item_render( OW_Event $event )
{
    $params = $event->getParams();
    $data = $event->getData();
    $language = OW::getLanguage();

    if ( $params['action']['entityType'] != 'board-topic' )
    {
        return;
    }

    $service = BOARD_BOL_BoardService::getInstance();
    $postCount = $service->findTopicPostCount($params['action']['entityId']) - 1;

    if ( !$postCount )
    {
        return;
    }

    if ( is_array($data['toolbar']) )
    {
        $data['toolbar'][] = array(
            'label' => $language->text('board', 'feed_toolbar_replies', array('postCount' => $postCount))
        );
    }

    $event->setData($data);

    $postIds = array();
    foreach ( $params['activity'] as $activity )
    {
        if ( $activity['activityType'] == 'board-post' )
        {
            $postIds[] = $activity['data']['postId'];
        }
    }

    if ( empty($postIds) )
    {
        return;
    }

    $postDto = null;
    foreach ( $postIds as $pid )
    {
        $postDto = $service->findPostById($pid);
        if ( $postDto !== null )
        {
            break;
        }
    }

    if ( $postDto === null )
    {
        return;
    }

    $postUrl = $service->getPostUrl($postDto->topicId, $postDto->id);
    $postUrlEmbed = '...';
    $content = UTIL_String::truncate(strip_tags($postDto->text), 100, $postUrlEmbed);
    $usersData = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($postDto->userId), true, true, true, false);

    $avatarData = $usersData[$postDto->userId];
    $postUrl = $service->getPostUrl($postDto->topicId, $postDto->id);

    $ipcContent = OW::getThemeManager()->processDecorator('mini_ipc', array(
            'avatar' => $avatarData, 'profileUrl' => $avatarData['url'], 'displayName' => $avatarData['title'], 'content' => $content));

    $data['assign']['activity'] = array('template' => 'activity', 'vars' => array(
            'title' => $language->text('board', 'feed_activity_last_reply', array('postUrl' => $postUrl)),
            'content' => $ipcContent
        ));

    $event->setData($data);
}
OW::getEventManager()->bind('feed.on_item_render', 'board_feed_on_item_render');

function board_feed_collect_configurable_activity( BASE_CLASS_EventCollector $event )
{
    $language = OW::getLanguage();
    $event->add(array(
        'label' => $language->text('board', 'feed_content_label'),
        'activity' => 'create:board-topic'
    ));

    $event->add(array(
        'label' => $language->text('board', 'feed_content_replies_label'),
        'activity' => 'board-post:board-topic'
    ));
}
OW::getEventManager()->bind('feed.collect_configurable_activity', 'board_feed_collect_configurable_activity');

function board_subscribe_user( OW_Event $e )
{
    $params = $e->getParams();
    $userId = (int) $params['userId'];
    $topicId = (int) $params['topicId'];

    if ( !$userId || ! $topicId )
    {
        return false;
    }

    $service = BOARD_BOL_SubscriptionService::getInstance();

    if ( $service->isUserSubscribed($userId, $topicId) )
    {
        return true;
    }

    $subs = new BOARD_BOL_Subscription();
    $subs->userId = $userId;
    $subs->topicId = $topicId;

    $service->addSubscription($subs);

    return true;
}

OW::getEventManager()->bind('board.subscribe_user', 'board_subscribe_user');

function board_feed_topic_like( OW_Event $event )
{
    $params = $event->getParams();

    if ( $params['entityType'] != 'board-topic' )
    {
        return;
    }

    $service = BOARD_BOL_BoardService::getInstance();
    $topic = $service->findTopicById($params['entityId']);
    $userId = $topic->userId;
    
    $userName = BOL_UserService::getInstance()->getDisplayName($userId);
    $userUrl = BOL_UserService::getInstance()->getUserUrl($userId);
    $userEmbed = '<a href="' . $userUrl . '">' . $userName . '</a>';

    OW::getEventManager()->trigger(new OW_Event('feed.activity', array(
        'activityType' => 'like',
        'activityId' => $params['userId'],
        'entityId' => $params['entityId'],
        'entityType' => $params['entityType'],
        'userId' => $params['userId'],
        'pluginKey' => 'photo'
    ), array(
        'string' => OW::getLanguage()->text('board', 'feed_activity_topic_string_like', array(
            'user' => $userEmbed
        ))
    )));
}
OW::getEventManager()->bind('feed.after_like_added', 'board_feed_topic_like');
