<?php

/**
 * EXHIBIT A. Common Public Attribution License Version 1.0
 * The contents of this file are subject to the Common Public Attribution License Version 1.0 (the “License”);
 * you may not use this file except in compliance with the License. You may obtain a copy of the License at
 * http://www.oxwall.org/license. The License is based on the Mozilla Public License Version 1.1
 * but Sections 14 and 15 have been added to cover use of software over a computer network and provide for
 * limited attribution for the Original Developer. In addition, Exhibit A has been modified to be consistent
 * with Exhibit B. Software distributed under the License is distributed on an “AS IS” basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the specific language
 * governing rights and limitations under the License. The Original Code is Oxwall software.
 * The Initial Developer of the Original Code is Oxwall Foundation (http://www.oxwall.org/foundation).
 * All portions of the code written by Oxwall Foundation are Copyright (c) 2011. All Rights Reserved.

 * EXHIBIT B. Attribution Information
 * Attribution Copyright Notice: Copyright 2011 Oxwall Foundation. All rights reserved.
 * Attribution Phrase (not exceeding 10 words): Powered by Oxwall community software
 * Attribution URL: http://www.oxwall.org/
 * Graphic Image as provided in the Covered Code.
 * Display of Attribution Information is required in Larger Works which are defined in the CPAL as a work
 * which combines Covered Code or portions thereof with code not governed by the terms of the CPAL.
 */

/**
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow_utilities
 * @since 1.0
 */
class UTIL_Url
{
    private static $redirectCodes = array(
        100 => "HTTP/1.1 100 متابعة",
        101 => "HTTP/1.1 101 تبديل البروتوكولات",
        200 => "HTTP/1.1 200 حسنا",
        201 => "HTTP/1.1 201 تم الإنشاء",
        202 => "HTTP/1.1 202 تم القبول",
        203 => "HTTP/1.1 203 معلومات غير موثوقة",
        204 => "HTTP/1.1 204 لا يوجد محتوى",
        205 => "HTTP/1.1 205 إعادة تعيين المحتوى",
        206 => "HTTP/1.1 206 محتوى جزئي",
        300 => "HTTP/1.1 300 اختيارات متعددة",
        301 => "HTTP/1.1 301 منقول بشكل دائم",
        302 => "HTTP/1.1 302 موجود",
        303 => "HTTP/1.1 303 إطلع على شيء آخر",
        304 => "HTTP/1.1 304 غير معدل",
        305 => "HTTP/1.1 305 إستخدام روكسي",
        307 => "HTTP/1.1 307 إعادة توجيه مؤقتة",
        400 => "HTTP/1.1 400 طلب سيئ",
        401 => "HTTP/1.1 401 غير مصرح به",
        402 => "HTTP/1.1 402 الدفعة مطلوبة",
        403 => "HTTP/1.1 403 محظور",
        404 => "HTTP/1.1 404 غير موجود",
        405 => "HTTP/1.1 405 أسلوب غير مسموح به",
        406 => "HTTP/1.1 406 غير مقبول",
        407 => "HTTP/1.1 407 مصادقة بروكسي مطلوبة",
        408 => "HTTP/1.1 408 طلب مهلة",
        409 => "HTTP/1.1 409 تضارب",
        410 => "HTTP/1.1 410 ذهب",
        411 => "HTTP/1.1 411 المدة المطلوبة",
        412 => "HTTP/1.1 412 الشرط المسبق فشل",
        413 => "HTTP/1.1 413 كيان طلب كبير جدا",
        414 => "HTTP/1.1 414 طلب - URI كبير جدا",
        415 => "HTTP/1.1 415 نوع الوسائط غير معتمد",
        416 => "HTTP/1.1 416 المجال المطلوب غير مشبع",
        417 => "HTTP/1.1 417 التوقع فشل",
        500 => "HTTP/1.1 500 خطأ خادم داخلي",
        501 => "HTTP/1.1 501 لم تنفذ",
        502 => "HTTP/1.1 502 بوابة غير صالحة",
        503 => "HTTP/1.1 503 الخدمة غير متوفرة",
        504 => "HTTP/1.1 504 مهلة البوابة"
    );

    /**
     * Makes search engines friendly redirect to provided URL.
     * 
     * @param string $url
     * @param integer $redirectCode
     */
    public static function redirect( $url, $redirectCode = null )
    {
        $redirectCode = in_array((int)$redirectCode, self::$redirectCodes) ? (int)$redirectCode : 301;

        header(self::$redirectCodes[$redirectCode]);
        header("Location: " . $url);
        exit();
    }

    public static function selfUrl()
    {
        $s = (!empty($_SERVER["HTTPS"]) && ($_SERVER["HTTPS"] == "on") ) ? 's' : '';
        $serverProtocol = strtolower($_SERVER["SERVER_PROTOCOL"]);
        $protocol = substr($serverProtocol, 0, strpos($serverProtocol, '/')) . $s;

        $port = ($_SERVER["SERVER_PORT"] == "80") ? "" : (":" . $_SERVER["SERVER_PORT"]);

        return $protocol . "://" . $_SERVER['SERVER_NAME'] . $port . $_SERVER['REQUEST_URI'];
    }
}